Ccalmet_to_v5d.f

C This is a skeleton conversion program for converting your data format
C to VIS-5D's v5d format (which is directly read by vis5d).

C This version allows you to specify a different number of levels per
C variable, map projections, and vertical coordinate systems.

C You need to insert code to do two things:
C     1. Open your input file and read the header information, putting
C        that information into the required variables.  If your file
C        doesn't contain some of the needed values you can just assign
C        constants to the variables.
C     2. read your grid data into the array G for each timestep and
C        variable.

C Pay special attention to comments in UPPERCASE.  They give more
C information about what you have to do.

C The rest of the program will handle creating the v5d file, writing the
C grid data, and closing the file.

C Compile this program with the foo2_to_v5d.f.m makefile.
C Run this program with two arguments:  the name of your input file and
C the name of the v5d output file.



      program calmet_to_v5d
      EXTERNAL v5dwrite, v5dmcfile, v5dcreatesimple, v5dcreate
c --- Include parameter statements
      include 'params.puf'
c      implicit none
      include "v5df.h"
c
c --- Include common blocks
      include 'chemdat.puf'
      include 'filnam.puf'
      include 'flags.puf'
      include 'gen.puf'
      include 'outpt.puf'
      include 'grid.puf'
      include 'methd.puf'
c
      integer iargc
c     Local vars
c      integer n
      real*4 G(MAXROWS, MAXCOLUMNS, MAXLEVELS), tmp1(mxnxy)
      character*100 inname, outname
      integer maxnl
      integer i

C  THE FOLLOWING VARIABLES DESCRIBE THE DATASET TO BE CONVERTED.  YOU
C  MUST INITIALIZE ALL THESE VARIABLES WITH VALUES FROM YOUR FILE OR
C  ASSIGN SUITABLE CONSTANTS.  SEE THE README FILE FOR DESCRIPTIONS
C  OF THESE VARIABLES.
      integer nr, nc, nl(MAXVARS)
      integer numtimes
      integer numvars
      character*10 varname(MAXVARS)
      integer dates(MAXTIMES)
      integer times(MAXTIMES)
      integer compressmode
      integer projection
      real proj_args(100)
      integer vertical
      real vert_args(MAXLEVELS)
      logical lprt


c     initialize the variables to missing values
      data nr,nc / IMISSING, IMISSING /
      data (nl(i),i=1,MAXVARS) / MAXVARS*IMISSING /
      data numtimes,numvars / IMISSING, IMISSING /
      data (varname(i),i=1,MAXVARS) / MAXVARS*"          " /
      data (dates(i),i=1,MAXTIMES) / MAXTIMES*IMISSING /
      data (times(i),i=1,MAXTIMES) / MAXTIMES*IMISSING /
      data compressmode / 1 /
      data projection / IMISSING /
      data (proj_args(i),i=1,100) / 100*MISSING /
      data vertical / IMISSING /
      data (vert_args(i),i=1,100) / 100*MISSING /

      data nw1/mxnxy/

c     get command line arguments
      if (iargc() .ne. 2) then
         print *,"Error:  two filename arguments are needed."
         call exit(1)
      else
         call getarg(1, inname)
         call getarg(2, outname)
      endif
      print 10,"Input file: ", inname
      print 10,"Output file: ", outname
 10   format (A,A)

C  OPEN YOUR DATAFILE (inname) HERE AND READ ITS HEADER INFORMATION TO
C  INITIALIZE THE ABOVE VARIABLES.

c ---    open CALMET meteorological data file (CALMET.DAT)
      open(io27,file=inname,status='old',form='unformatted')

      lprt=.true.
      call met1(lprt,tmp1,nw1,irtype)
      nr = ny
      nc = nx
c
c -- assign calpuff values to vis5d variable names
      numtimes = irlg + 1
      numargs = 8
c
      projection = 0
      proj_args(1) = yorig + dgrid*(nr-1)
      proj_args(2) = xorig
      proj_args(3) = dgrid
c  to get correct direction of coordinate for UTM (affects u-component)
      proj_args(4) = -dgrid
      compressmode = 1
c
      vertical = 2
      do k = 1,nz
         vert_args(k) = 0.001*zface(k+1)
      enddo
c
      nl(1) = nz
      nl(2) = nz
      nlindx = 3
      if(lcalgrd) then
         nl(nlindx) = nz
         nlindx = nlindx+1
         if(irtype.eq.1) then
            nl(nlindx) = nz
            nlindx = nlindx+1
         endif
      endif
      if(irtype.eq.1) then
         nl(nlindx) = 1
         nlindx = nlindx+1
         nl(nlindx) = 1
         nlindx = nlindx+1
         nl(nlindx) = 1
         nlindx = nlindx+1
         nl(nlindx) = 1
         nlindx = nlindx+1
         nl(nlindx) = 1
         nlindx = nlindx+1
         nl(nlindx) = 1
         nlindx = nlindx+1
         nl(nlindx) = 1
      endif
c
      nowyr = ibyr
      nowmo = ibmo
      nowdy = ibdy
      nowhr = ibhr - 1
      do it = 0,irlg
         call datepp(nowhr,nowdy,nowmo,nowyr)
         jjj = julian(nowdy,nowmo,nowyr) - julian(1,1,nowyr) + 1
         dates(it+1) = 1000*nowyr + jjj
         times(it+1) = 10000*nowhr
      enddo
c
      varname(1) = 'U'
      varname(2) = 'V'
      iargs = 2
      if(lcalgrd) then
         iargs = iargs + 1
         varname(iargs) = 'W'
         if(irtype.eq.1) then
            iargs = iargs + 1
            varname(iargs) = 'T'
         endif
      endif
      if(irtype.eq.1) then
         iargs = iargs + 1
         varname(iargs) = 'PG-Stab'
         iargs = iargs + 1
         varname(iargs) = 'U*'
         iargs = iargs + 1
         varname(iargs) = 'Zi'
         iargs = iargs + 1
         varname(iargs) = 'L'
         iargs = iargs + 1
         varname(iargs) = 'W*'
         iargs = iargs + 1
         varname(iargs) = 'PP'
         iargs = iargs + 1
         varname(iargs) = 'TERRAIN'
      endif
C  
C  
      numvars = iargs

      nr = ny
      nc = nx
      print *,'nr,nc,nzm:  ',nr,nc,nzm
c     Create the v5d file.
      n = v5dcreate( outname, numtimes, numvars, nr, nc, nl,
     *               varname, times, dates, compressmode,
     *               projection, proj_args, vertical, vert_args )
      if (n .eq. 0) then
         call exit(1)
      endif

C  YOU MAY CALL v5dsetlowlev OR v5dsetunits HERE.  SEE README FILE.

      maxnl = nl(1)
      do i=1,numvars
         if (nl(i) .gt. maxnl) then
            maxnl = nl(i)
         endif
      enddo

c     enter the conversion subroutine
      call convert(nw1,nr,nc,nl,maxnl,numtimes,numvars,G,irtype,
     .             proj_args(4))

c     close the v5d file and exit
      n = v5dclose()
      if (n .eq. 0) then
c        failed
         call exit(1)
      else
c        success
         call exit(0)
      endif

      end

c-----------------------------------------------------------------------
      subroutine convert(nw1,nr,nc,nl,maxnl,numtimes,numvars,G,irtype,
     .                   proj_args4)
c       Read 3-D grids from input file, write them to the output file.
c-----------------------------------------------------------------------
c      implicit none
c      EXTERNAL v5dwrite
      include 'params.puf'
      include 'grid.puf'
      include 'gen.puf'
      include 'methr.puf'
      include 'methd.puf'
      include "v5df.h"
c     Arguments
      integer nr
      integer nc
      integer nl(MAXVARS)
      integer maxnl
      integer numtimes
      integer numvars
c     Local vars
      integer it, iv
      real tmp1(mxnxy),proj_args4
c      dimension G(MAXROWS, MAXCOLUMNS, MAXLEVELS)
      dimension G(nr,nc,MAXLEVELS)
      logical ldbmet
c
      data ldbmet/.false./

      do it=1,numtimes
c
c         do iv=1,numvars
c

C  READ YOUR DATA FOR TIME STEP NUMBER IT AND VARIABLE NUMBER IV INTO
C  THE ARRAY G HERE.  NOTE THAT G(1,1,1) IS THE NORTH WEST BOTTOM CORNER
C  AND G(NR,NC,NL) IS THE SOUTH EAST TOP CORNER.  ALSO, THE LOOPS CAN
C  BE CHANGED TO WORK IN ANY ORDER, THE v5dwrite CALL WILL STILL WORK.
C  VALUES GREATER THEN 1.0E30 ARE CONSIDERED TO BE 'MISSING'.
c-----------------------------------------------------------------------
        call RDMET(tmp1,nw1,ldbmet,umet,vmet,wmet,tmet,ipgt,htmix,ustar,
     1              xmonin,wstar,rmm,tempss,rhoss,qswss,irhss,ipcode,
     2              kdathr,irtype)
         iv = 1
c  Since proj_args(4)<0 must reverse u-coordinate
         if(proj_args4.lt.0.0) then
            call reverse_u(umet,nl(iv))
            print *,"proj_args4 = ",proj_args4
         endif
         call convertR3(it,iv,nr,nc,nl(iv),G,umet)
         iv = 2
         call convertR3(it,iv,nr,nc,nl(iv),G,vmet)
         if(lcalgrd) then
            iv = iv+1
            call convertR3(it,iv,nr,nc,nl(iv),G,wmet)
            if(irtype.eq.1) then
               iv = iv+1
               call convertR3(it,iv,nr,nc,nl(iv),G,tmet)
            endif
         endif
         if(irtype.eq.1) then
            iv = iv+1
            call convertI2(it,iv,nr,nc,nl(iv),G,ipgt)
            iv = iv+1
            call convertR2(it,iv,nr,nc,nl(iv),G,ustar)
            iv = iv+1
            call convertR2(it,iv,nr,nc,nl(iv),G,z0)
            iv = iv+1
            call convertR2(it,iv,nr,nc,nl(iv),G,xmonin)
            iv = iv+1
            call convertR2(it,iv,nr,nc,nl(iv),G,wstar)
            iv = iv+1
            call convertR2(it,iv,nr,nc,nl(iv),G,rmm)
            iv = iv+1
            call convertR2(it,iv,nr,nc,nl(iv),G,elev)
         endif
      enddo
      return
      end
c-----------------------------------------------------------------------
      subroutine reverse_u(umet,nliv)
c-----------------------------------------------------------------------
      include 'params.puf'
      dimension umet(mxnx,mxny,mxnz)
c
      do i=1,mxnx
         do j=1,mxny
            umet(i,j,nliv) = -umet(i,j,nliv)
         enddo
      enddo
      end
c-----------------------------------------------------------------------
      subroutine convertR3(it,iv,nr,nc,nzm,G,u)
c-----------------------------------------------------------------------
c
      EXTERNAL v5dwrite
      include 'params.puf'
      include "v5df.h"
c      dimension g(MAXROWS, MAXCOLUMNS, MAXLEVELS), u(mxnx,mxny,mxnz)
      dimension G(nr,nc,nzm), u(mxnx,mxny,mxnz)
      integer it,iv,nr,nc,nzm
      do ir = 1,nr
         jr = nr-ir+1
         do ic = 1,nc
            do k = 1,nzm
               G(ir,ic,k) = u(ic,jr,k)
            enddo
         enddo
      enddo
c      print *,'convertR3 ',it,iv,u(1,1,1),G(nr,1,1)
c     write the 3-D grid to the v5d file
      n = v5dwrite( IT, IV, G )
      if (n .eq. 0) then
c        error
         print *,'Error return from w5dwrite in convertR3'
         call exit(1)
      endif
      return
      end
c-----------------------------------------------------------------------
      subroutine convertI3(it,iv,nr,nc,nzm,G,mat)
c-----------------------------------------------------------------------
c
      EXTERNAL v5dwrite
      include 'params.puf'
      include "v5df.h"
      dimension mat(mxnx,mxny,mxnz),G(nr,nc,nzm)
      integer it,iv,nr,nc,nzm
      do ir = 1,nr
         jr = nr-ir+1
         do ic = 1,nc
            do k=1,nzm
               G(ir,ic,k) = float(mat(ic,jr,k))
            enddo
         enddo
      enddo
c      print *,'convertI3 ',it,iv,mat(1,1,1),G(nr,1,1)
c     write the 3-D grid to the v5d file
      n = v5dwrite( IT, IV, G )
      if (n .eq. 0) then
c        error
         print *,'Error return from w5dwrite in convertI3'
         call exit(1)
      endif
      return
      end
c-----------------------------------------------------------------------
      subroutine convertR2(it,iv,nr,nc,nzm,G,u)
c-----------------------------------------------------------------------
c
      EXTERNAL v5dwrite
      include 'params.puf'
      include "v5df.h"
      dimension G(nr,nc,nzm), u(mxnx,mxny)
      integer it,iv,nr,nc,nzm
      k = 1
      do ir = 1,nr
         jr = nr-ir+1
         do ic = 1,nc
            G(ir,ic,k) = u(ic,jr)
         enddo
      enddo
c      print *,'convertR2 ',it,iv,u(1,1),G(nr,1,1)
c     write the 3-D grid to the v5d file
      n = v5dwrite( IT, IV, G )
      if (n .eq. 0) then
c        error
         print *,'Error return from w5dwrite in convertR2'
         call exit(1)
      endif
      return
      end
c-----------------------------------------------------------------------
      subroutine convertI2(it,iv,nr,nc,nzm,G,mat)
c-----------------------------------------------------------------------
c
      EXTERNAL v5dwrite
      include 'params.puf'
      include "v5df.h"
      dimension mat(mxnx,mxny),G(nr,nc,nzm)
      integer it,iv,nr,nc,nzm
      k = 1
      do ir = 1,nr
         jr = nr-ir+1
         do ic = 1,nc
            G(ir,ic,k) = float(mat(ic,jr))
         enddo
      enddo
c      print *,'convertI2 ',it,iv,mat(1,1),G(nr,1,1)
c     write the 3-D grid to the v5d file
      n = v5dwrite( IT, IV, G )
      if (n .eq. 0) then
c        error
         print *,'Error return from w5dwrite in convertI2'
         call exit(1)
      endif
      return
      end
c----------------------------------------------------------------------
      subroutine met1(lecho,wrk1,nwork,irtype)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 4.0       Level: 900228                  MET1
c                J. Scire, SRC
c
c --- PURPOSE:  Read the 14 header records of the CALMET meteorological
c               data file
c
c --- INPUTS:
c            NX - integer       - Number of X cells in meteorological
c                                 grid
c            NY - integer       - Number of Y cells in meteorological
c                                 grid
c         LECHO - logical       - Control variable for output of header
c                                 record information
c   WRK1(nwork) - real array    - Work array of length "NWORK" words
c         NWORK - integer       - Dimension of work arrays -- NOTE:
c                                 NWORK must be at least as large as
c                                 nx*ny
c     Parameters:
c           MXNZP1, MXSS, MXUS, MXPS, IO6
c
c --- OUTPUT:
c     Common block /METHD/ variables:
c           NXM, NYM, NZM, XGRIDM, XORIGM, YORIGM, IUTMZNM,
c           NSSTA, NPSTA, NLU, IWAT1, IWAT2, ZFACEM(mxnzp1),
c           XSSTA(mxss),YSSTA(mxss),XUSTA(mxus),YUSTA(mxus),
c           XPSTA(mxps),YPSTA(mxps),Z0(mxnx,mxny),ILANDU(mxnx,mxny),
c           ELEV(mxnx,mxny),XLAI(mxnx,mxny),NEARS(mxnx,mxny),
c           LCALGRD
c
c --- MET1 called by:  SETUP
c --- MET1 calls:      RDR1D, RDR2D, RDI2D, METQA, OUT
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.puf'
      include 'lon.puf'
c
      real wrk1(nwork)
      integer iwrk1(mxnxy),idum(mxnx,mxny)
      character*80 title(3)
      character*70 messag
      character*8 vermet,levmet,clab1,clab2
      character*8 clabexp,clabex1,clabex2
      logical ldate,lecho
c
c --- Include common block /METHD/ -- CALMET header record data
      include 'methd.puf'
      include 'gen.puf'
      include 'grid.puf'
c
      data ldate/.false./
c
c --- read CALMET header records
c
      lecho = .true.
c --- record #1 - run title -- 60 words
      read(io27)title
      if(LECHO)then
         write(io6,102)title
102      format(//1x,13('----------')//
     1   1x,'Data read from header records of MET output ',
     2   'file'//3(1x,a80/))
      endif
c -------------------------------------
c
c --- record #2 - run control parameters -- 26 words
c --- (vermet, levmet are both 8 bytes)
      read(io27)vermet,levmet,ibyr,ibmo,ibdy,ibhr,ibtz,irlg,irtype,
     1 nx, ny, nz, xgrid, xorig, yorig, iutmzn, iwfcod, nssta,
     2 nusta, npsta, nowsta, nlu, iwat1, iwat2, lcalgrd
      if(LECHO)then
         write(io6,*)
         write(io6,*)
         write(io6,*)
         write(io6,104)vermet,levmet
104      format(1x,'CALMET Version: ',a8,3x,'Level: ',a8)
         write(io6,*)
         write(io6,*)' IBYR    = ',ibyr
         write(io6,*)' IBMO    = ',ibmo
         write(io6,*)' IBDY    = ',ibdy
         write(io6,*)' IBHR    = ',ibhr
         write(io6,*)' IBTZ    = ',ibtz
         write(io6,*)' IRLG    = ',irlg
         write(io6,*)' IRTYPE  = ',irtype
         write(io6,*)' LCALGRD = ',lcalgrd
         write(io6,*)' NXM     = ',nx
         write(io6,*)' NYM     = ',ny
         write(io6,*)' NZM     = ',nz
         write(io6,*)' XGRIDM  = ',xgrid
         write(io6,*)' XORIGM  = ',xorig
         write(io6,*)' YORIGM  = ',yorig
         write(io6,*)' IUTMZNM = ',iutmzn
         write(io6,*)' IWFCOD  = ',iwfcod
         write(io6,*)' NSSTA   = ',nssta
         write(io6,*)' NUSTA   = ',nusta
         write(io6,*)' NPSTA   = ',npsta
         write(io6,*)' NOWSTA  = ',nowsta
         write(io6,*)' NLU     = ',nlu
         write(io6,*)' IWAT1   = ',iwat1
         write(io6,*)' IWAT2   = ',iwat2
      endif
      dgrid = 0.001*xgrid
      xorig = 0.001*xorig
      yorig = 0.001*yorig
      nxm = nx
      nym = ny
      nzm = nz
c
c --- Check that enough workspace has been allocated
      nec=nx*ny
      if(nwork.lt.nec)then
         write(io6,*)'ERROR in subr. MET1 -- insufficient workspace ',
     1   'allocated to work arrays -- Allocated (NWORK) = ',nwork,
     2   '   Required (NEC) = ',nec
         stop
      endif
c
      nzmp1=nz+1
      if(mxnzp1.lt.nzmp1)then
         write(io6,*)'ERROR in subr. MET1 -- dimension of ZFACEM ',
     1   'array is too small for no. layers in MET file'
         write(io6,*)'MXNZP1 = ',mxnzp1,' NZMP1 = ',nzmp1
         stop
      endif

c --- Add new record #3 starting with Level=980304
c
c -------------------------------------
c
c --- record #3 - additional run control parameters -- 8 words
      read(io27) xlat0,xlon0,llconf,conec,xlat1,xlat2,rlat0,rlon0
c -------------------------------------
c
c --- record #3 - cell face heights (NZ + 1 words)
      call rdr1d(io27,zface,nzmp1,clab1,ndathr)
c
      if(LECHO)then
         write(io6,203)(zface(n),n=1,nzmp1)
203      format(1x,'ZFACE  = ',10(f6.0,', ')/10x,10(f6.0,', '))
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ZFACE')then
         clabexp='ZFACE'
         go to 999
      endif
c -------------------------------------
c
c ************************************************
c --- Perform QA checks on header record variables
c ************************************************
      call metqa
c
c --- records #4 & 5 - x, y coordinates of surface stations
c --- (NSSTA words each record)
      if(nssta.gt.0)then
         call rdr1d(io27,xssta,nssta,clab1,ndathr)
         call rdr1d(io27,yssta,nssta,clab2,ndathr)
         if(LECHO)then
            write(io6,*)clab1,' = ',(xssta(n),n=1,nssta)
            write(io6,*)clab2,' = ',(yssta(n),n=1,nssta)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XSSTA'.or.clab2.ne.'YSSTA')then
            clabex1='XSSTA'
            clabex2='YSSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- records #6 & 7 - x, y coordinates of upper air stations
c --- (NUSTA words each record)
      if(nusta.gt.0)then
         call rdr1d(io27,xusta,nusta,clab1,ndathr)
         call rdr1d(io27,yusta,nusta,clab2,ndathr)
         if(LECHO)then
            write(io6,*)clab1,' = ',(xusta(n),n=1,nusta)
            write(io6,*)clab2,' = ',(yusta(n),n=1,nusta)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XUSTA'.or.clab2.ne.'YUSTA')then
            clabex1='XUSTA'
            clabex2='YUSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- records #8 & 9 - x, y coordinates of precipitation stations
c --- (NPSTA words each record)
      if(npsta.gt.0)then
         call rdr1d(io27,xpsta,npsta,clab1,ndathr)
         call rdr1d(io27,ypsta,npsta,clab2,ndathr)
         if(LECHO)then
            write(io6,*)clab1,' = ',(xpsta(n),n=1,npsta)
            write(io6,*)clab2,' = ',(ypsta(n),n=1,npsta)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XPSTA'.or.clab2.ne.'YPSTA')then
            clabex1='XPSTA'
            clabex2='YPSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- record #10 - surface roughness lengths (NX * NY words)
      call rdr2d(io27,z0,wrk1,mxnx,mxny,nx,ny,clab1,ndathr)
      if(LECHO)then
         messag='Surface roughness lengths (m)'
         messag(63:70)=clab1
         print *,messag
c         call out(z0,idum,1,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'Z0')then
         clabexp='Z0'
         go to 999
      endif
c -------------------------------------
c
c --- record #11 - land use categories (NX * NY words)
      call rdi2d(io27,ilandu,iwrk1,mxnx,mxny,nx,ny,clab1,ndathr)
      if(LECHO)then
         messag='Land use categories'
         messag(63:70)=clab1
         print *,messag
c         call out(xdum,ilandu,2,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ILANDU')then
         clabexp='ILANDU'
         go to 999
      endif
c -------------------------------------
c
c --- record #12 - elevations (NX * NY words)
      call rdr2d(io27,elev,wrk1,mxnx,mxny,nx,ny,clab1,ndathr)
      if(LECHO)then
         messag='Terrain heights (m)'
         messag(63:70)=clab1
         call out(elev,idum,1,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ELEV')then
         clabexp='ELEV'
         go to 999
      endif
c -------------------------------------
c
c --- record #13 - leaf area index (NX * NY words)
      call rdr2d(io27,xlai,wrk1,mxnx,mxny,nx,ny,clab1,ndathr)
      if(LECHO)then
         messag='Leaf area index'
         messag(63:70)=clab1
c         call out(xlai,idum,1,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'XLAI')then
         clabexp='XLAI'
         go to 999
      endif
c -------------------------------------
c
c --- record #14 - nearest surface station to each grid point
c                  (NX * NY words)
      call rdi2d(io27,nears,iwrk1,mxnx,mxny,nx,ny,clab1,ndathr)
      if(LECHO)then
         messag='Nearest surface station no. to each grid point'
         messag(63:70)=clab1
         print *,messag
c         call out(xdum,nears,2,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'NEARS')then
         clabexp='NEARS'
         go to 999
      endif
c -------------------------------------
c
      return
c
c --- Write error messages -- incorrect record label read
999   continue
      write(io6,1001)clab1,clabexp
1001  format(/1x,'ERROR in subr. MET1 -- incorrect record label ',
     1 'read from MET data file'//1x,'Label read     = ',a8/
     2                               1x,'Label expected = ',a8)
      stop
1999  continue
      write(io6,2001)clab1,clab2,clabex1,clabex2
2001  format(/1x,'ERROR in subr. MET1 -- incorrect record label ',
     1 'read from MET data file'//
     2 1x,'Labels read     = ',a8,2x,a8/
     3 1x,'Labels expected = ',a8,2x,a8)
      stop
c
      end
c----------------------------------------------------------------------
      subroutine metqa
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 4.0       Level: 900228                 METQA
c                J. Scire, SRC
c
c --- PURPOSE:  Perform consistency checks of data read from the
c               CALMET header records with control file inputs
c
c --- INPUTS:
c     Common block /GRID/ variables:
c           NX, NY, NZ, DGRID, XORIG, YORIG, IUTMZN, ZFACE(mxnzp1)
c     Common block /METHD/ variables:
c           NXM, NYM, NZM, XGRIDM, XORIGM, YORIGM, IUTMZNM,
c           NSSTA, NUSTA, NPSTA, ZFACEM(mxnzmp1)
c     Parameters:
c           MXNZP1, MXNZMP1, MXSS, MXUS, MXPS, IO6
c
c --- OUTPUT:
c         none
c
c --- METQA called by:  MET1
c --- METQA calls:      none
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.puf'
c
c --- Include common blocks
      include 'grid.puf'
      include 'methd.puf'
c
c --- Initialize error flag
      ierr=0
c
c --- Vertical face heights
      nzp1=nz+1
c
c
c
c --- Stop execution if any errors encountered
      if(ierr.eq.1)STOP
c
      return
      end
c----------------------------------------------------------------------
      subroutine rdmet(wrk1,nwork,lecho,umet,vmet,wmet,tmet,ipgt,htmix,
     1 ustar,xmonin,wstar,rmm,tempss,rhoss,qswss,irhss,ipcode,kdathr,
     2 irtype)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 4.0       Level: 900228                 RDMET
c                J. Scire, SRC
c
c --- PURPOSE:  Read CALMET meteorological data for one hour
c
c --- INPUTS:
c   WRK1(nwork) - real array    - Work array of length "NWORK" words
c         NWORK - integer       - Dimension of work array -- NOTE:
c                                 NWORK must be at least as large as
c                                 the max. of (NXM * NYM or NSSTA)
c         LECHO - logical       - Flag controlling printing of input
c                                 meteorological data fields
c    Common block /METHD/ variables:
c         NXM, NYM, NZM, NSSTA, NPSTA, LCALGRD
c    Parameters:
c         MXNZMP1, IO6
c
c --- OUTPUT:
c
c   UMET(mxnx,mxny,mxnz) - real    - U component of the wind (m/s)
c                                    at each grid point
c   VMET(mxnx,mxny,mxnz) - real    - V-component of the wind (m/s)
c   WMET(mxnx,mxny,mxnz) - real    - W-component of the wind (m/s)
c   TMET(mxnx,mxny,mxnz) - real    - 3-D temperature field (deg. K)
c        IPGT(mxnx,mxny) - integer - PGT stability class
c       HTMIX(mxnx,mxny) - real    - Mixing height (m)
c       USTAR(mxnx,mxny) - real    - Surface friction velocity (m/s)
c      XMONIN(mxnx,mxny) - real    - Monin-Obukhov length (m)
c       WSTAR(mxnx,mxny) - real    - Convective velocity scale (m/s)
c         RMM(mxnx,mxny) - real    - Precipitation rate (mm/hr)
c          TEMPSS(nssta) - real    - Air temperature (deg. K) at
c                                    each surface met. station
c           RHOSS(nssta) - real    - Air density (kg/m**3)
c           QSWSS(nssta) - real    - Short wave solar radiation
c                                    (W/m**2)
c           IRHSS(nssta) - integer - Relative humidity (percent)
c          IPCODE(nssta) - integer - Precipitation code at each surface
c                                    station
c                 KDATHR - integer - Date of hour of data (YYJJJHH)
c
c --- RDMET called by:  COMP
c --- RDMET calls:      RDR1D, RDR2D, RDI1D, RDI2D, OUT
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.puf'
c
c --- Include common /METHD/ -- met. data parameters
      include 'methd.puf'
c
      real umet(mxnx,mxny,mxnz),vmet(mxnx,mxny,mxnz)
      real tmet(mxnx,mxny,mxnz),wmet(mxnx,mxny,mxnz)
      real htmix(mxnx,mxny),ustar(mxnx,mxny),xmonin(mxnx,mxny)
      real wstar(mxnx,mxny),rmm(mxnx,mxny)
      real wrk1(nwork),xdum(mxnx,mxny)
      integer iwrk1(mxnxy),idum(mxnx,mxny)
      real tempss(nssta),rhoss(nssta),qswss(nssta)
c
      integer ipgt(mxnx,mxny),irhss(nssta),ipcode(nssta)
c
      character*70 messag
      character*8 clabel
      character*8 clabexp
c
      logical lecho,ldate
c
      data ldate/.false./
c
c --- Check that enough workspace has been allocated
      nec=max0(nxm*nym,nssta)
      if(nwork.lt.nec)then
         write(io6,*)'ERROR in subr. RDMET -- insufficient workspace ',
     1   'allocated to work arrays -- Allocated (NWORK) = ',nwork,
     2   '   Required (NEC) = ',nec
         stop
      endif
c
c --- read the U, V wind components
      kdathr=0
      do 10 iz=1,nzm
      call rdr2d(io27,umet(1,1,iz),wrk1,mxnx,mxny,nxm,nym,clabel,
     1 ndathr1)
c
c --- Check that record label matches expected label
      clabexp='U-LEV'
      write(clabexp(6:8),'(i3)')iz
      if(clabel.ne.clabexp)go to 999
c
      call rdr2d(io27,vmet(1,1,iz),wrk1,mxnx,mxny,nxm,nym,clabel,
     1 ndathr2)
      clabexp(1:1)='V'
      if(clabel.ne.clabexp)go to 999
      ioffset = 0
      joffset = 0
c      if(iz.eq.3)then
         do jfoo=1,nym
            write(*,*) (umet(ifoo+ioffset,jfoo+joffset,iz),ifoo=1,nxm)
            write(*,*) 'U above, V below '
            write(*,*) (vmet(ifoo+ioffset,jfoo+joffset,iz),ifoo=1,nxm)
            print *,'this line was blank ',jfoo,' ',iz
         enddo
c      endif
c
c --- Skip the W wind component (conditional)
      if(lcalgrd)then
         call rdr2d(io27,wmet(1,1,iz),wrk1,mxnx,mxny,nxm,nym,clabel,
     1              ndathr3)
         clabexp(1:5)='WFACE'
         if(clabel.ne.clabexp)go to 999
         if(ndathr2.ne.ndathr3)go to 2999
      endif
c
c --- Check for consistency of date/time
      if(ndathr1.ne.ndathr2)go to 2999
      if(kdathr.eq.0)then
c ---    first vertical level
         kdathr=ndathr1
      else if(ndathr1.ne.kdathr)then
c ---    date/time does not match value for previous layer
         ndathr=ndathr1
         go to 3999
      endif
10    continue
c
c --- Read the 3-D temperature field
      if(lcalgrd.and.irtype.eq.1)then
         clabexp='T-LEV'
         do 12 iz=1,nzm
         call rdr2d(io27,tmet(1,1,iz),wrk1,mxnx,mxny,nxm,nym,clabel,
     1    ndathr)
c
         write(clabexp(6:8),'(i3)')iz
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
12       continue
      endif
c
c --- read other 2-D meteorological fields
c
      if(irtype.eq.1) then
c --- PGT stability class
         call rdi2d(io27,ipgt,iwrk1,mxnx,mxny,nxm,nym,clabel,ndathr)
         clabexp='IPGT'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
c --- FRICTION VELOCITY
         call rdr2d(io27,ustar,wrk1,mxnx,mxny,nxm,nym,clabel,ndathr)
         clabexp='USTAR'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
c --- MIXING HEIGHT
         call rdr2d(io27,htmix,wrk1,mxnx,mxny,nxm,nym,clabel,ndathr)
         clabexp='ZI'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
c --- MONIN-OBUKHOV LENGTH
         call rdr2d(io27,xmonin,wrk1,mxnx,mxny,nxm,nym,clabel,ndathr)
         clabexp='EL'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
c --- CONVECTIVE VELOCITY SCALE
         call rdr2d(io27,wstar,wrk1,mxnx,mxny,nxm,nym,clabel,ndathr)
         clabexp='WSTAR'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
c --- PRECIPITATION DATA
         if(npsta.gt.0)then
            call rdr2d(io27,rmm,wrk1,mxnx,mxny,nxm,nym,clabel,ndathr)
         endif
      endif
c
c --- read 1-D meteorological fields:
c        Air temp. (deg. K),
c        Air density (kg/m**3),
c        Short-wave solar radiation (W/m**2),
c        Relative humidity (percent),
c        Precipitation code
c
c
      if(irtype.eq.1) then
         call rdr1d(io27,tempss,nssta,clabel,ndathr)
         clabexp='TEMPK'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
         call rdr1d(io27,rhoss,nssta,clabel,ndathr)
         clabexp='RHO'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
         call rdr1d(io27,qswss,nssta,clabel,ndathr)
         clabexp='QSW'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
         call rdi1d(io27,irhss,nssta,clabel,ndathr)
         clabexp='IRH'
         if(clabel.ne.clabexp)go to 999
         if(ndathr.ne.kdathr)go to 3999
c
c --- Precipitation code at surface stations
         if(npsta.gt.0)then
            call rdi1d(io27,ipcode,nssta,clabel,ndathr)
            clabexp='IPCODE'
            if(clabel.ne.clabexp)go to 999
            if(ndathr.ne.kdathr)go to 3999
         endif
      endif
c
c --- Write the input fields, if requested
      if(LECHO)then
c
c ---    Date and time
         write(io6,*)
         write(io6,*)'------ MET. FIELDS FOR DATE/TIME = ',kdathr,
     1   ' (YYJJJHH) ------'
         write(io6,*)
c
c ---    3-D fields
         do 101 iz=1,nzm
         messag='U-component (m/s) -- Level: '
         write(messag(29:31),'(i3)')iz
         call out(umet(1,1,iz),idum,1,5,ldate,messag,nxm,nym)
         messag(1:1)='V'
         call out(vmet(1,1,iz),idum,1,5,ldate,messag,nxm,nym)
101      continue
c
c ---    2-D fields
         messag='PGT stability class'
         call out(xdum,ipgt,2,1,ldate,messag,nxm,nym)
         messag='Friction velocity (m/s)'
         call out(ustar,idum,1,5,ldate,messag,nxm,nym)
         messag='Mixing height (m)'
         call out(htmix,idum,1,5,ldate,messag,nxm,nym)
         messag='Monin-Obukhov length (m)'
         call out(xmonin,idum,1,5,ldate,messag,nxm,nym)
         messag='Convective velocity scale (m/s)'
         call out(wstar,idum,1,5,ldate,messag,nxm,nym)
         if(npsta.gt.0)then
            messag='Precipitation rate (mm/hr)'
            call out(rmm,idum,1,5,ldate,messag,nxm,nym)
         endif
c
c ---    1-D fields
         write(io6,*)'TEMPSS (deg. K) = ',(tempss(n),n=1,nssta)
         write(io6,*)'RHOSS (kg/m**3)  = ',(rhoss(n),n=1,nssta)
         write(io6,*)'QSWSS (W/m**2)   = ',(qswss(n),n=1,nssta)
         write(io6,*)'IRHSS (percent)  = ',(irhss(n),n=1,nssta)
         if(npsta.gt.0)write(io6,*)'IPCODE           = ',
     1    (ipcode(n),n=1,nssta)
      endif
c
      return
c
c --- Write error messages -- incorrect record label read
999   continue
      write(io6,1001)clabel,clabexp
1001  format(/1x,'ERROR in subr. RDMET -- incorrect record label ',
     1 'read from MET data file'//1x,'Label read     = ',a8/
     2                            1x,'Label expected = ',a8)
      print *,'mxnx,mxny,nxm,nym,date ',mxnx,mxny,nxm,nym,ndathr1
      stop
c
c --- date/time variables do not match
2999  continue
      write(io6,*)'ERROR in subr. RDMET -- date/time variables ',
     1 'do not match -- NDATHR1 = ',ndathr1,' NDATHR2 = ',ndathr2,
     2 ' NDATHR3 = ',ndathr3
      stop
3999  continue
      write(io6,*)'ERROR in subr. RDMET -- date/time variables ',
     1 'do not match -- NDATHR = ',ndathr,' KDATHR = ',kdathr
      stop
      end
c----------------------------------------------------------------------
      subroutine rdi1d(iomet,idat,nwords,clabel,ndathr)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 4.0       Level: 940430                   RDI1D
c                J. Scire, SRC
c
c --- PURPOSE:  Read "NWORDS" of a one-dimensional integer array
c
c --- INPUTS:
c         IOMET - integer       - Fortran unit number of input file
c
c --- OUTPUT:
c  IDAT(nwords) - integer array - Array read from file
c        NWORDS - integer       - Number of words to read
c        CLABEL - character*8   - Variable name
c        NDATHR - integer       - Date and time of data (YYJJJHH)
c
c --- RDI1D called by:  RDMET
c --- RDI1D calls:      none
c----------------------------------------------------------------------
c
      integer idat(nwords)
      character*8 clabel
c
      read(iomet,IOSTAT=ios,ERR=999,END=998)clabel,ndathr,idat
      print *,' --in rdi1d:  clabel:  ',clabel,ndathr
      return
 999  print *,'Read error = ',ios,' in rdi1d; clabel = ',clabel
      stop
 998  print *,'EOF encountered in rdi1d, execution stopped'
      stop
      end
c----------------------------------------------------------------------
      subroutine rdi2d(iomet,idat,ibuf,mxnx,mxny,nx,ny,clabel,ndathr)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 4.0       Level: 940430                 RDI2D
c                J. Scire, SRC
c
c --- PURPOSE:  Read NX * NY words of a 2-D integer array
c
c --- INPUTS:
c               IOMET - integer       - Fortran unit number of input
c                                       file
c         IBUF(nx,ny) - integer array - Buffer to hold input data
c           MXNX,MXNY - integers      - Dimensions of data array
c               NX,NY - integers      - Actual size of grid to read
c
c --- OUTPUT:
c     IDAT(mxnx,mxny) - integer array - Input data array (padded if
c                                       necessary)
c              CLABEL - character*8   - Variable name
c              NDATHR - integer       - Date and time of data (YYJJJHH)
c
c --- RDI2D called by:  MET1, RDMET
c --- RDI2D calls:      none
c----------------------------------------------------------------------
c      include 'params.puf'
      integer idat(mxnx,mxny),ibuf(nx,ny)
      character*8 clabel
c
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- read full grid
         read(iomet,IOSTAT=ios,ERR=999,END=998)clabel,ndathr,idat
      else
c
c ---    only a portion of grid being used -- read and
c ---    transfer from buffer
c
         read(iomet,IOSTAT=ios,ERR=999,END=998)clabel,ndathr,ibuf
c
         do 10 i=1,nx
         do 10 j=1,ny
         idat(i,j)=ibuf(i,j)
10       continue
      endif
c
      print *,' --in rdi2d:  clabel:  ',clabel,ndathr
      return
 999  print *,'Read error = ',ios,' in rdi2d; clabel = ',clabel
      stop
 998  print *,'EOF encountered in rdi2d, execution stopped'
      stop
      end
c----------------------------------------------------------------------
      subroutine rdr1d(iomet,x,nwords,clabel,ndathr)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 4.0       Level: 940430                 RDR1D
c                J. Scire, SRC
c
c --- PURPOSE:  Read "NWORDS" of a one-dimensional real array
c
c --- INPUTS:
c         IOMET - integer     - Fortran unit number of input file
c
c --- OUTPUT:  
c     X(nwords) - real array  - Array read from file
c        NWORDS - integer     - Number of words to read
c        CLABEL - character*8 - Variable name
c        NDATHR - integer     - Date and time of data (YYJJJHH)
c
c --- RDR1D called by:  MET1, RDMET
c --- RDR1D calls:      none
c----------------------------------------------------------------------
      real x(nwords)
      character*8 clabel
c
      read(iomet,IOSTAT=ios,ERR=999,END=998)clabel,ndathr,x
      print *,' --in rdr1d:  clabel:  ',clabel,ndathr
      return
 999  print *,'Read error = ',ios,' in rdr1d; clabel = ',clabel
      stop
 998  print *,'EOF encountered in rdr1d, execution stopped'
      stop
      end
c----------------------------------------------------------------------
      subroutine rdr2d(iomet,x,xbuf,mxnx,mxny,nx,ny,clabel,ndathr)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 4.0       Level: 940430                 RDR2D
c                J. Scire, SRC
c
c --- PURPOSE:  Read NX * NY words of a 2-D real array
c
c --- INPUTS:
c            IOMET - integer     - Fortran unit number of input file
c      XBUF(nx,ny) - real array  - Buffer to hold input data
c      MXNX,MXNY   - integers    - Dimensions of data array
c            NX,NY - integers    - Actual size of grid to read
c           CLABEL - character*8 - Variable name
c           NDATHR - integer     - Date and time of data (YYJJJHH)
c
c --- OUTPUT:
c     X(mxnx,mxny) - real array  - Input data array (padded if nec.)
c
c --- RDR2D called by:  MET1, RDMET
c --- RDR2D calls:      none
c----------------------------------------------------------------------
c      include 'params.puf'
      real x(mxnx,mxny),xbuf(nx,ny)
      character*8 clabel
c
c      print *,'--in rdr2d:  nx,ny,mxnx,mxny = ',nx,ny,mxnx,mxny
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- read full grid
         read(iomet,IOSTAT=ios,ERR=999,END=998)clabel,ndathr,x
      else
c
c ---    only a portion of grid being used -- read and
c ---    transfer from buffer
         read(iomet,IOSTAT=ios,ERR=999,END=998)clabel,ndathr,xbuf
c
         do 10 i=1,nx
         do 10 j=1,ny
         x(i,j)=xbuf(i,j)
10       continue
      endif
      print *,' --in rdr2d:  clabel:  ',clabel,ndathr
c
      return
 999  print *,'Read error = ',ios,' in rdr2d; clabel = ',clabel
      stop
 998  print *,'EOF encountered in rdr2d, execution stopped'
      stop
      end
c-----------------------------------------------------------------------
      subroutine datepp(ihr,idy,imn,iyr)
c
c -- Given the arguments of hour, day, month, and year will return
c -- the next hour after making adjustments for midnight, end of month
c -- and the New Year.  Arguments are modified.
c
c-----------------------------------------------------------------------
      dimension months(12,2)
c-----------------------------------------------------------------------
      data months /31,28,31,30,31,30,31,31,30,31,30,31
     .            ,31,29,31,30,31,30,31,31,30,31,30,31/
c-----------------------------------------------------------------------
c
c -- check for leap year
c
      if(mod(iyr,4).eq.0) then
         ileap = 2
      else
         ileap = 1
      endif
c
      ihr = ihr + 1
      if(ihr.gt.23) then
         ihr = 0
         idy = idy + 1
         if(idy.gt.months(imn,ileap)) then
            idy = 1
            imn = imn + 1
            if(imn.gt.12) then
               imn = 1
               iyr = iyr + 1
            endif
         endif
      endif
      return
      end
c-----------------------------------------------------------------------
      function julian(idy,imo,iyr)
c
      localmo = imo
      if(iyr.lt.50) then
         localyr = iyr + 2000
      elseif(iyr.gt.49.and.iyr.lt.100) then
         localyr = iyr + 1900
      else
         localyr = iyr
      endif
      if(imo.le.2) then
         localyr = localyr - 1
         localmo = localmo + 12
      endif
c
      ia = int(localyr/100)
      ib = 2 - ia + int(ia/4)
      ia1 = int(365.25*float(4716+localyr))
      ia2 = int(30.6001*float(1+localmo))
      julian = ia1+ia2 + idy + ib - 1524
      return
      end
c----------------------------------------------------------------------
      subroutine out(rarray,iarray,ityp,nsigd,ldate,messag,nx,ny)
c----------------------------------------------------------------------
c
c --- CALPUFF  Version: 4.0       Level: 940430                     OUT
c ---          J. Scire, SRC
c
c --- PURPOSE:  Write a gridded field of real or integer numbers
c
c --- INPUTS:
c     RARRAY(MXNX,MXNY) - Real array  - Array of real numbers to print
c                                       (used only if ITYP = 1)
c     IARRAY(MXNX,MXNY) - Int. array  - Array of integer numbers to
c                                       print (used only if ITYP = 2)
c                  ITYP - Integer     - Array type (1=real, 2=integer)
c                 NSIGD - Integer     - No. digits to print (valid range
c                                       for NSIGD is 1 to 5)
c                 LDATE - Logical     - Control variable for printing
c                                       of date (.true. = print date in
c                                       common /GEN/, .false. = do not
c                                       print date)
c                MESSAG - Char.*70    - Label of table
c                    NX - Integer     - No. X grid cells being used in
c                                       array
c                    NY - Integer     - No. Y grid cells being used in
c                                       array
c       Common block /DATEHR/ variables:
c          NYR, NMO, NDAY, NJUL, NHR  - (Used only if LDATE=.true.)
c       Parameters: MXNX, MXNY, IO6
c
c --- OUTPUT:  none
c
c --- OUT    called by:  MET1
c --- OUT    calls:      WRT, WRT2
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.puf'
c
c --- Include date/hour common block
      include 'datehr.puf'
c
      real rarray(mxnx,mxny)
c
      integer iarray(mxnx,mxny),icol(5)
      integer iout(mxnx)
c
      logical ldate
c
      character*70 messag
      character*1 sign(mxnx),plus,minus
      character*24 form1(5)
      character*21 form2(5)
      character*18 form3(5)
c
      data icol /40,40,30,25,20/
      data plus,minus /'+','-'/
      data form1 /'(1x,i2,1x,1hI,40(i2,1x))',
     1            '(1x,i2,1x,1hI,40(i2,1x))',
     2            '(1x,i2,1x,1hI,40(i3,1x))',
     3            '(1x,i2,1x,1hI,40(i4,1x))',
     4            '(1x,i2,1x,1hI,40(i5,1x))'/
      data form2 /'(4x,1hI,40(1x,a1,1x))',
     1            '(4x,1hI,40(1x,a1,1x))',
     2            '(4x,1hI,40(2x,a1,1x))',
     3            '(4x,1hI,40(3x,a1,1x))',
     4            '(4x,1hI,40(4x,a1,1x))'/
      data form3 /'(5x,40(i2,1x))',
     1            '(5x,40(i2,1x))',
     2            '(5x,40(i3,1x))',
     3            '(5x,40(i4,1x))',
     4            '(5x,40(i5,1x))'/
c
c --- check that valid values of array type (ityp) and print digits
c --- (nsigd) have been passed to routine
      if(ityp.ne.1.and.ityp.ne.2)then
         write(io6,*)'ERROR in SUBR. OUT -- invalid value of ITYP -- ',
     1   'ITYP = ',ityp
         stop
      endif
      if(nsigd.lt.1.or.nsigd.gt.5)then
         write(io6,*)'ERROR in SUBR. OUT -- invalid value of NSIGD -- ',
     1   'NSIGD = ',nsigd
         stop
      endif
c
      icr=2
      if(nsigd.eq.1)icr=1
      if(mod(nx,icol(nsigd)).eq.0)then
         npass=nx/icol(nsigd)
      else
         npass=nx/icol(nsigd)+1
      endif
c
c --- real array -- find min. & max. values
      if(ityp.ne.1)go to 50
      xmax=-1.e-25
      xmin=1.e25
      do 10 i=1,nx
      do 10 j=1,ny
      if(rarray(i,j).gt.xmax)xmax=rarray(i,j)
      if(rarray(i,j).lt.xmin)xmin=rarray(i,j)
10    continue
      if(xmin.ne.0.0.or.xmax.ne.0.0)go to 12
      if(ldate)write(io6,94)messag,nyr,nmo,nday,njul,nhr
      if(.not.ldate)write(io6,95)messag
      write(io6,11)
11    format(1x,'GRID NOT PRINTED -- all values zero')
      return
c
12    continue
      xexp=xmax
      if(abs(xmin).gt.xmax)xexp=abs(xmin)
      iexp=int(alog10(xexp))
      if(xexp.lt.1.0)iexp=iexp-1
      nexp=iexp-(nsigd-icr)
      xscale=10.**(-nexp)
c
      ic2=0
      do 30 ipass=1,npass
c
      if(ldate)write(io6,94)messag,nyr,nmo,nday,njul,nhr
94    format(/1x,a70,2x,'year: ',i2,2x,'month: ',i2,2x,'day: ',i2,2x,
     1 'Julian day: ',i3,2x,'hour(0-23): ',i2/)
      if(.not.ldate)write(io6,95)messag
95    format(/1x,a70/)
      write(io6,109)nexp
109   format(1x,'Multiply all values by 10 ** ',i3/)
c
      ic1=ic2+1
      ic2=ic2+icol(nsigd)
      if(ic2.gt.nx)ic2=nx
c
         do 20 jj=ny,1,-1
         icnt=0
c
            do 18 i=ic1,ic2
            icnt=icnt+1
            if(rarray(i,jj).lt.0)then
               iout(icnt)=-int((rarray(i,jj)*xscale-0.5))
               sign(icnt)=minus
            else
               iout(icnt)=int(rarray(i,jj)*xscale+0.5)
               sign(icnt)=plus
            endif
18          continue
         call wrt(form1(nsigd),form2(nsigd),jj,iout,sign,icnt,io6)
20       continue
      nund=(nsigd+1)*icnt-1
      if(nsigd.eq.1)nund=(nsigd+2)*icnt-1
      write(io6,101)(minus,n=1,nund)
101   format(5x,128a1)
      call wrt2(form3(nsigd),ic1,ic2,io6)
30    continue
      return
c
c --- integer array -- find min. & max. values
50    continue
      kmax=-9999999
      kmin=9999999
      do 110 i=1,nx
      do 110 j=1,ny
      if(iarray(i,j).gt.kmax)kmax=iarray(i,j)
      if(iarray(i,j).lt.kmin)kmin=iarray(i,j)
110   continue
      if(kmin.ne.0.or.kmax.ne.0)go to 102
      if(ldate)write(io6,94)messag,nyr,nmo,nday,njul,nhr
      if(.not.ldate)write(io6,95)messag
      write(io6,11)
      return
c
102   continue
      xexp=kmax
      if(iabs(kmin).gt.kmax)xexp=iabs(kmin)
      iexp=int(alog10(xexp))
      if(xexp.lt.1.0)iexp=iexp-1
      nexp=iexp-(nsigd-icr)
      xscale=10.**(-nexp)
c
      ic2=0
      do 130 ipass=1,npass
c
      if(ldate)write(io6,94)messag,nyr,nmo,nday,njul,nhr
      if(.not.ldate)write(io6,95)messag
      write(io6,109)nexp
c
      ic1=ic2+1
      ic2=ic2+icol(nsigd)
      if(ic2.gt.nx)ic2=nx
c
         do 120 jj=ny,1,-1
         icnt=0
c
            do 118 i=ic1,ic2
            icnt=icnt+1
            if(iarray(i,jj).lt.0)then
               iout(icnt)=-int((iarray(i,jj)*xscale-0.5))
               sign(icnt)=minus
            else
               iout(icnt)=int(iarray(i,jj)*xscale+0.5)
               sign(icnt)=plus
            endif
118         continue
         call wrt(form1(nsigd),form2(nsigd),jj,iout,sign,icnt,io6)
120      continue
      nund=(nsigd+1)*icnt-1
      if(nsigd.eq.1)nund=(nsigd+2)*icnt-1
      write(io6,101)(minus,n=1,nund)
      call wrt2(form3(nsigd),ic1,ic2,io6)
130   continue
c
      return
      end
c----------------------------------------------------------------------
      subroutine wrt(form1,form2,jj,iout,sign,n,io6)
c----------------------------------------------------------------------
c
c --- CALPUFF  Version: 4.0       Level: 940430                     WRT
c ---          J. Scire, SRC
c
c --- PURPOSE:  Write one Y row of gridded data
c
c --- INPUTS:
c              FORM1 - Char.*24    - Format field for Y label and data
c                                    to be printed
c              FORM2 - Char.*21    - Format field for sign of data
c                 JJ - Integer     - Y grid cell number
c            IOUT(N) - Int. array  - Array of data to be printed
c                                    (one Y row)
c            SIGN(N) - Char.*1     - Array containing sign of data
c                                    ('+' or '-')
c                  N - Integer     - Number of cells in this row
c                IO6 - Integer     - Fortran unit no. of output
c
c --- OUTPUT:  none
c
c --- WRT called by:  OUT, OUTSAM
c --- WRT calls:      none
c----------------------------------------------------------------------
      integer iout(n)
c
      character*1 sign(n)
      character*24 form1
      character*21 form2
c
      write(io6,form1)jj,iout
      write(io6,form2)sign
c
      return
      end
c----------------------------------------------------------------------
      subroutine wrt2(form,n1,n2,io6)
c----------------------------------------------------------------------
c
c --- CALPUFF  Version: 4.0       Level: 940430                    WRT2
c ---          J. Scire, SRC
c
c --- PURPOSE:  Write a line labeling grid cell numbers
c
c --- INPUTS:
c               FORM - Char.*18    - Format field of data to be printed
c                 N1 - Integer     - Starting grid cell number
c                 N2 - Integer     - Ending grid cell number
c                IO6 - Integer     - Fortran unit no. of output
c
c --- OUTPUT:  none
c
c --- WRT2 called by:  OUT, OUTSAM
c --- WRT2 calls:      none
c----------------------------------------------------------------------
      character*18 form
c
      write(io6,form)(i,i=n1,n2)
      return
      end
